/*
 * ClipLibraryManager.java
 * Copyright (c) 1999-2000 Andre Kaplan, Chris Laird
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.util.Enumeration;
import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;

import org.gjt.sp.jedit.jEdit;

import org.gjt.sp.util.Log;


public class ClipLibraryManager
    extends    OrderedProperties
{
    private static final String CLIP_LIBRARY_PROPERTY = "clipLibrary.";
    private static final String CLIP_LIBRARY_DIRECTORY = "clipLibraryDirectory.";

    private static ClipLibraryManager instance = null;

    public static ClipLibraryManager getInstance() {
        if (instance == null) {
            instance = new ClipLibraryManager();
        }
        return instance;
    }


    public static ClipLibrary getCurrentClipLibrary() {
        String currentClipLib = jEdit.getProperty(
            ClipperPlugin.PROPERTY_PREFIX + ClipperPlugin.CURRENT_CLIPLIBRARY
        );

        ClipLibrary cl = null;

        if (currentClipLib != null) {
            cl = (ClipLibrary)ClipLibraryManager.getInstance().get(currentClipLib);
        }

        return cl;
    }


    private ClipLibraryManager() {
        super(true, false);
        this.loadClipLibrariesFromProps();
        this.scanClipLibraryDirs();
    }


    public ClipLibrary getClipLibrary(int index) {
        if (this.isEmpty()) {
            return null;
        }
        return (ClipLibrary) getValue(index);
    }


    private void scanClipLibraryDirs() {
        String propValue;
        int prop = 0;
        do {
            propValue = jEdit.getProperty(
                ClipperPlugin.PROPERTY_PREFIX + CLIP_LIBRARY_DIRECTORY + prop
            );
            if ((propValue != null) && (propValue.length() > 0)) {
                // Log.log(Log.DEBUG, this, "cliplib " + prop + " = " + propValue);
                this.loadClipLibrariesFromDir(propValue);
            }
            prop++;
        }
        while (propValue != null);
    }


    private void loadClipLibrariesFromDir(String dirName) {
        // Append a File.separator to the directory name if it doesn't already have one
        if (!dirName.endsWith(File.separator)) {
            dirName = dirName.concat(File.separator);
        }
        File dir = new File(dirName);
        String[] files = dir.list(new ClipLibraryFilenameFilter());
        //if files is null, a directory with the specified name was not found
        //report the error and exit
        if (files == null) {
            Log.log(Log.MESSAGE, this,
                  "Could not find the directory \""
                + dir + "\" to scan for .cliplibrary files"
            );

            return;
        }

        Log.log(Log.MESSAGE, this,
              "Scanning directory \""
            + dir + "\" for .cliplibrary files"
        );

        for (int i = 0; i < files.length; i++) {
            String clipfile = dirName + files[i];
            try {
                this.loadClipLibrary(new FileInputStream(clipfile));
            }
            catch (IOException e) {
                Log.log(Log.NOTICE, this,
                    "Could not load clip library \"" + clipfile + "\""
                );
            }
        }
    }


    /**
     * Load all clip libraries specified by properties.
     */
    private void loadClipLibrariesFromProps() {
        String propValue;
        int prop = 0;
        do {
            propValue = jEdit.getProperty(
                ClipperPlugin.PROPERTY_PREFIX + CLIP_LIBRARY_PROPERTY + prop
            );
            if (propValue != null) {
                //Log.log(Log.DEBUG,this,"cliplib " + prop + " = " + propValue);
                try {
                    this.loadClipLibrary(
                        ClipperPlugin.class.getResourceAsStream(propValue)
                    );
                }
                catch (IOException e) {
                    Log.log(Log.NOTICE, this,
                          "Could not load clip library \""
                        + propValue + "\" from plugin jar"
                    );
                }
//HACK: JARClassLoader seems to throw a NullPointerException if the requested
//      resource doesn't exist. So catch any exception here, not just IOException.
                catch (Exception e) {
                    Log.log(Log.WARNING, this,
                          "Could not load clip library \""
                        + propValue + "\" from plugin jar. Exception follows"
                    );
                    Log.log(Log.WARNING, this, e);
                }
            }
            prop++;
        }
        while (propValue != null);
    }


    private void loadClipLibrary(InputStream is)
        throws IOException
    {
        ClipLibrary cl = new ClipLibrary(is);
        this.put(cl.getTitle(), cl);
        Log.log(Log.MESSAGE, this,
            "Loaded clip library \"" + cl.getTitle() + "\""
        );
    }


    private static class ClipLibraryFilenameFilter implements FilenameFilter {
        public boolean accept(File dir, String name) {
            return name.endsWith(".cliplibrary");
        }
    }
}
