/*
 * Clipper.java
 * Copyright (c) 1999-2000 Andre Kaplan, Chris Laird
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.StringTokenizer;

import javax.swing.*;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.textarea.JEditTextArea;
import org.gjt.sp.util.Log;


/**
 * Visual clipper panel
**/
public class Clipper extends JPanel
{
    private static ClipLibraryManager clipLibraryManager = null;


    public static Clipper createClipper(View view) {
        if (clipLibraryManager == null) {
            clipLibraryManager = ClipLibraryManager.getInstance();
        }

        ClipLibrary cl = clipLibraryManager.getCurrentClipLibrary();
        if (cl == null) {
            cl = clipLibraryManager.getClipLibrary(0);
        }

        Clipper clipper = new Clipper(view, cl);
        clipper.setCurrentClipLibrary(cl);

        return clipper;
    }


    private View             view;
    private ClipLibrary      clipLibrary;
    private JList            list;
    private JComboBox        comboBox;
    private ListListener     listListener;
    private ComboBoxListener comboBoxListener;


    private Clipper(View view, ClipLibrary cl) {
        this.setLayout(new BorderLayout());
        setMinimumSize(new Dimension(50,20));
        setBorder(BorderFactory.createEtchedBorder());
        this.view = view;
        this.clipLibrary = cl;

        //setup list
        this.list = new JList(new ClipListModel(cl));
        this.list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.listListener = new ListListener();
        this.list.addMouseListener(this.listListener);
        this.list.registerKeyboardAction(
            this.listListener,
            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            WHEN_ANCESTOR_OF_FOCUSED_COMPONENT
        );
        this.list.registerKeyboardAction(
            this.listListener,
            KeyStroke.getKeyStroke(' '),
            WHEN_ANCESTOR_OF_FOCUSED_COMPONENT
        );

        this.comboBox = new JComboBox(new ClipLibraryComboBoxModel());
        this.comboBoxListener = new ComboBoxListener();
        this.comboBox.addActionListener(this.comboBoxListener);
        this.add(this.comboBox, BorderLayout.NORTH);
        this.add(new JScrollPane(this.list), BorderLayout.CENTER);
    }


    public ClipLibrary getCurrentClipLibrary() {
        return this.clipLibrary;
    }


    public void setCurrentClipLibrary(ClipLibrary cl) {
        this.comboBox.setSelectedItem(cl.getTitle());
    }


    /**
     * Remove listeners and perform general cleanup.
     */
    public void dispose() {
        //remove listeners
        this.list.removeMouseListener(this.listListener);
        this.list.resetKeyboardActions();
        this.comboBox.removeActionListener(this.comboBoxListener);
    }


    private void insertClip(int index) {
        String label = clipLibrary.getKey(index);
        this.insertClip(label);
    }


    /**
     * Insert a text clip in the active buffer. The clip to insert is found by
     * querying the current clipper clip library
    **/
    private void insertClip(String label) {
        Buffer buffer = this.view.getBuffer();
        //if the buffer is read only, clips can't be inserted
        if (buffer.isReadOnly()) {
            this.view.getToolkit().beep();
            return;
        }
        JEditTextArea textArea = view.getTextArea();

        //retrieve the text to insert in the buffer
        Clipper clipper =
             (Clipper) this.view
            .getDockableWindowManager()
            .getDockableWindow(ClipperPlugin.DOCKABLE_NAME)
            .getComponent();
        ClipLibrary cl = clipper.getCurrentClipLibrary();
        String text = (String) cl.get(label);

        //retrieve the selection from the text area (if anY)
        String selection = textArea.getSelectedText();
        if (selection == null) {
            selection = "";
        }
        //remember the final destination of the cursor
        //(-1 means don't move it)
        int cursorDestination = -1;

        buffer.beginCompoundEdit();

        String lineSeparator = "\n";  // jEdit.getProperty("buffer.lineSeparator");

        //break the text clip up into lines
        StringTokenizer st = new StringTokenizer(text, "\n\r");
        while (st.hasMoreTokens()) {
            String line = st.nextToken();
            // Log.log(Log.DEBUG, this, "line=[" + line + "]");

            //search the text to find a cursor marker
            boolean foundCursorMarker = false;
            int foundAt = 0;

            // Insert a newline after each line of the text clip unless this
            // is the last line
            String finalNewline = (st.hasMoreTokens() ? lineSeparator : "");

            while (!foundCursorMarker && (foundAt != -1)) {
                //search for an occurrence of the cursor marker char
                foundAt = line.indexOf(ClipperPlugin.CURSOR_MARKER, foundAt);
                // Log.log(Log.DEBUG, this, "found cursor marker at [" + foundAt + "]");
                if (foundAt != -1) {
                    //if the next char is not also a cursor marker (i.e. it is not escaped)
                    //we've found one
                    // Log.log(Log.DEBUG, this, "char at [" + (foundAt+1) + "] is [" + line.charAt(foundAt - 1) + "]");
                    if ((foundAt == line.length()-1) || (line.charAt(foundAt + 1) != ClipperPlugin.CURSOR_MARKER)) {
                        foundCursorMarker = true;
                    } else {
                        //if this is an escaped cursor marker, remove the dummy
                        //char and skip over the next char
                        line = line.substring(0, foundAt) + line.substring(foundAt + 1);
                        // Log.log(Log.DEBUG, this, "line is now [" + line + "]");
                        foundAt++;
                    }
                }
            }

            //if a cursor marker wasn't found, just replace the current
            //selection with the new text clip
            if (!foundCursorMarker) {
                textArea.setSelectedText(line);
            } else {
                // insert the clip line with the selection in the marked cursor position
                textArea.setSelectedText(
                      line.substring(0, foundAt)
                    + selection
                    + line.substring(foundAt + 1)
                );
            }

            buffer.indentLine(textArea.getSelectionStartLine(), true, true);

            if (foundCursorMarker) {
                // remember where the cursor should be positioned when we've
                // finished inserting the text clip
                cursorDestination =
                      textArea.getCaretPosition()
                    - (line.length() - (foundAt + 1));
            }

            // insert the line break
            textArea.setSelectedText(finalNewline);
        }

        //if a cursor marker was found and no text was selected, move the caret
        //to the position of the cursor marker
        if ((cursorDestination != -1) && (selection.equals(""))) {
            textArea.setCaretPosition(cursorDestination);
        }

        buffer.endCompoundEdit();

        //set the keyboard focus back to the text area in the view window,
        //ready for more editing
        this.view.toFront();
        this.view.getEditPane().focusOnTextArea();
    }


    /**
     * Private class that acts as a listener to the list of clips and inserts
     * a clip either on double-click or when space is hit.
     */
    private class ListListener
        extends    MouseAdapter
        implements ActionListener
    {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                int index = Clipper.this.list.getSelectedIndex();
                if (index != -1) {
                    Clipper.this.insertClip(index);
                }
            }
        }


        public void actionPerformed(ActionEvent e) {
            int index = Clipper.this.list.getSelectedIndex();
            if (index != -1) {
                Clipper.this.insertClip(index);
            }
        }
    }


    /**
     * Private class that acts as a listener to the combobox and changes the
     * the listmodel to reflect the selected cliplibrary.
     */
    private class ComboBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Clipper.this.clipLibrary =
                Clipper.this.clipLibraryManager.getClipLibrary(
                    Clipper.this.comboBox.getSelectedIndex()
                );
            Clipper.this.list.setModel(
                new ClipListModel(Clipper.this.clipLibrary)
            );
            jEdit.setProperty(
                  ClipperPlugin.PROPERTY_PREFIX
                + ClipperPlugin.CURRENT_CLIPLIBRARY,
                clipLibrary.getTitle()
            );
        }
    }
}
