/*
 * OrderedProperties.java
 * Copyright (c) 1999-2000 Andre Kaplan, Chris Laird
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.util.Properties;
import java.util.Vector;

import org.gjt.sp.util.Log;


/**
 * A Properties subclass that keeps an index of its keys, so that they
 * can be returned in a set order. Also provides support for retrieving
 * keys in sorted order.
 *
 * @version $Id: OrderedProperties.java,v 1.3 2000/12/31 16:05:42 akaplan Exp $
 */
public class OrderedProperties extends Properties {

    //Constants referring to keys in a file
    private static final String TITLE = "@title";
    private static final String SORT_CLIPS = "@sortClips";
    private static final String CASE_SENSITIVE_SORT = "@caseSensitiveSort";


    private String title;
    private Vector sortedIndex;
    private boolean sortClips;
    private boolean caseSensitiveSort;


    /**
     * Construct a new <CODE>OrderedProperties</CODE>.
     */
    public OrderedProperties(boolean sorted, boolean caseSensitive) {
        super();
        sortedIndex = new Vector();
        sortClips = sorted;
        caseSensitiveSort = caseSensitive;
    }


    /**
     * Construct a new unsorted <CODE>OrderedProperties</CODE>.
     */
    public OrderedProperties() {
        this(false, false);
    }


    /**
     * Overrides put method in Properties to cast the key to a string
     * and always returns null.
     */
    public Object put(Object key, Object value) {
        putAndIndex((String) key, value);
        return null;
    }


    /**
     * Add a String key/value pair.
     */
    public void putAndIndex(String key, Object value) {
        //if this is a special property, don't add it to the list
        if (key.equals(TITLE)) {
            this.title = (String) value;
        }
        else if (key.equals(SORT_CLIPS)) {
            sortClips = ((String) value).equalsIgnoreCase("true");
        }
        else if (key.equals(CASE_SENSITIVE_SORT)) {
            caseSensitiveSort = ((String) value).equalsIgnoreCase("true");
        }
        //otherwise, add the key/value pair
        else {
            super.put(key, value);
            //if the list is to be sorted, insert it in the right place
            if (sortClips) {
                //insert the key in the sorted index
                boolean inserted = false;
                for (int i = 0; i < sortedIndex.size(); i++) {
                    String indexStr = (String) sortedIndex.elementAt(i);
                    String compKey = key;
                    if (!caseSensitiveSort) {
                        indexStr = indexStr.toLowerCase();
                        compKey = compKey.toLowerCase();
                    }
                    // Log.log(Log.DEBUG, this, "comparing [" + compKey + "] to [" + indexStr + "]");
                    if (indexStr.compareTo(compKey) > 0) {
                        sortedIndex.insertElementAt(key, i);
                        inserted = true;
                        // Log.log(Log.DEBUG, this, "inserted [" + key + "] at position [" + i + "]");
                        break;
                    }
                }
                // Log.log(Log.DEBUG, this, sortedIndex.toString());
                if (!inserted) {
                    sortedIndex.addElement(key);
                }
            }
            //if the clips aren't to be sorted, just add it at the end
            else {
                sortedIndex.addElement(key);
            }
        }
    }


    public String getValueAsString(String key) {
        return (String) get(key);
    }


    public String getKey(int index) {
        return (String) sortedIndex.elementAt(index);
    }


    public Object getValue(int index) {
        return get(getKey(index));
    }


    public String getTitle() {
        return title;
    }


    public void setTitle(String title) {
        this.title = title;
    }


    public Vector getSortedIndex() {
        return sortedIndex;
    }


    public String toString() {
        StringBuffer ret = new StringBuffer(this.getClass().getName()).append("[\n");
        for (int i = 0; i < sortedIndex.size(); i++) {
            Object label = sortedIndex.elementAt(i);
            ret.append("  ").append(label).append("=").append(get(label)).append("\n");
        }
        ret.append("]");
        return ret.toString();
    }
}


/*
 * $Log: OrderedProperties.java,v $
 * Revision 1.3  2000/12/31 16:05:42  akaplan
 * Clipper 0.9: updated for jEdit 3.0final
 *
 * Revision 1.2  2000/12/31 01:46:32  andre
 * Code cleanings
 *
 * Revision 1.1.1.1  2000/11/12 14:24:32  andre
 * Clipper 0.8 initial import
 *
 * Revision 1.2  2000/09/18 23:03:52  akaplan
 * Added Dockable support, removed PluginHolder support, adapted to jEdit 2.6pre6 changes, code cleanings
 *
 * Revision 1.1.1.1  2000/04/16 01:51:12  akaplan
 * Clipper Plugin initial import
 *
*/
